-- ************************************************************************** --
-- * TitanSaveInfo.lua                                                      * --
-- *                                                                        * --
-- *  By: Rothpada (Shadow Council)                                         * --
-- ************************************************************************** --

local TITAN_SAVE_INFO_ID = "TitanSaveInfo";
local AceTimer = LibStub("AceTimer-3.0");
local L = LibStub("AceLocale-3.0"):GetLocale("TitanSaveInfo", true)

		---------------
		-- CONSTANTS --
		---------------	

local TitanSaveInfo_IS_DEBUGGING =  ( false and GetRealmName() == "Shadow Council" )
                                or ( UnitName("player") == "Balther"
                                     and GetRealmName() == "Shadow Council" )
                                or ( UnitName("player") == "Rothpada"
                                    and GetRealmName() == "Broxigar" );
local TitanSaveInfo_LEVEL_THRESHOLD = 80;                                   

		---------------
		-- VARIABLES --
		---------------


		-------------
		-- STRUCTS --
		-------------

-- ************************************************************************** --
-- ******************************** Titan Functions ************************* --
-- ************************************************************************** --

function TitanPanelTitanSaveInfoButton_OnLoad(self)
	self.registry = {
		id = TITAN_SAVE_INFO_ID,
		menuText = L["TITAN_SAVE_INFO_MENU_TEXT"],
		buttonTextFunction = "TitanPanelTitanSaveInfoButton_GetButtonText",
		tooltipTitle = L["TITAN_SAVE_INFO_TOOLTIP_TITLE"],
		tooltipTextFunction = "TitanPanelTitanSaveInfoButton_GetTooltipText",
		savedVariables = {
			ShowLabelText = 1,
		}
	};	
	
	TitanSaveInfo_debug("Debugging output is turned on");
	
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("PLAYER_LEAVING_WORLD");
	self:RegisterEvent("PLAYER_LOGOUT");	
	self:RegisterEvent("PLAYER_QUITING");	
	self:RegisterEvent("WORLD_MAP_UPDATE");	
	self:RegisterEvent("VARIABLES_LOADED");		 	
	
	DEFAULT_CHAT_FRAME:AddMessage(TITAN_SAVE_INFO_ID.." by Rothpada (Shadow Council)", 1, 1, 1)
end 

function TitanPanelTitanSaveInfoButton_OnEvent(self, event, ...)
	if ( event == "PLAYER_ENTERING_WORLD" 
	        or event == "PLAYER_LEAVING_WORLD"
	        or event == "PLAYER_LOGOUT"
	        or event == "PLAYER_QUITING"
	        or event == "WORLD_MAP_UPDATE" ) then
	    TitanSaveInfo_refresh();
	elseif ( event == "VARIABLES_LOADED" ) then
		-- Initialize lock tables
		if ( not SaveData ) then 
			SaveData = {}; 
		end
		if ( UnitLevel("player") >= TitanSaveInfo_LEVEL_THRESHOLD
		        and ( not SaveData[GetRealmName()] ) ) then 
			SaveData[GetRealmName()] = {}; 
		end
		if ( UnitLevel("player") >= TitanSaveInfo_LEVEL_THRESHOLD 
		        and ( not SaveData[GetRealmName()][UnitName("player")] ) ) then 
			SaveData[GetRealmName()][UnitName("player")] = {}; 
		end
    end
end

function TitanPanelTitanSaveInfoButton_OnClick(self, button)
    TitanSaveInfo_refresh();
end

function TitanPanelTitanSaveInfoButton_GetButtonText(id)
	return L["TITAN_SAVE_INFO_BUTTON_LABEL"];
end

function TitanPanelTitanSaveInfoButton_GetTooltipText(self)
    TitanSaveInfo_refresh();
    local tooltip = "";
	
	if ( SaveData ) then
        for realm,players in pairs(SaveData) do
            tooltip = tooltip..TitanSaveInfo_getColouredText(realm, "daa520").."\n";
            for name,difficulties in pairs(players) do
                -- Colour the current player differently
                if ( name == UnitName("player") and realm == GetRealmName() ) then
                    tooltip = tooltip.."  "..TitanSaveInfo_getColouredText(name, "ffa500");
                else
                    tooltip = tooltip.."  "..TitanSaveInfo_getColouredText(name, "yellow");
                end	
                tooltip = tooltip..":";
                local subtip = "";
                local hasEntry = false;
                for difficulty,instances in pairs(difficulties) do
                    subtip = "";
                    hasEntry = false;
                    if ( instances ) then
                        subtip = subtip.."\n    "..TitanSaveInfo_getColouredText(difficulty, "white")..":";
                    end
                    for instance,expiry in pairs(instances) do
                        hasEntry = true;
                        if ( expiry > time() ) then
                            subtip = subtip.."\n      "..TitanSaveInfo_getColouredText(instance, "red");
                        end
                    end
                    if ( hasEntry ) then
                        tooltip = tooltip..subtip;              
                    end
                end
                tooltip = tooltip.."\n";
            end
        end
    end
    return tooltip;
end

function TitanPanelRightClickMenu_PrepareTitanSaveInfoMenu()

end

-- ************************************************************************** --
-- ******************************** TitanSaveInfo Functions ******************** --
-- ************************************************************************** --

function TitanSaveInfo_refresh()
    local realm = GetRealmName();
    local name = UnitName("player");
	if ( UnitLevel("player") >= TitanSaveInfo_LEVEL_THRESHOLD ) then		
		--SaveData[realm][name] = {};
	
		for i=1,GetNumSavedInstances() do
			local instance,_,resetTime,_,locked,_,_,_,_,difficulty = GetSavedInstanceInfo(i);
			if ( not SaveData[realm][name][difficulty] ) then
			    SaveData[realm][name][difficulty] = {};
            end
			if ( locked ) then
                SaveData[realm][name][difficulty][instance] = time() + resetTime;
            else
                SaveData[realm][name][difficulty][instance] = nil;
            end
		end
	end  
end

function TitanSaveInfo_debug(msg)
	if ( msg == nil ) then
		msg = "<nil>";
	end
	if TitanSaveInfo_IS_DEBUGGING then
		TitanDebug("TitanSaveInfo> "..msg);
	end
end

function TitanSaveInfo_getColouredText(msg, colour)
	local colourCode = "";
	
	if ( msg == nil ) then
		return "";
	end
		
	if ( colour == nil or colour == "default" ) then
		return msg;
	elseif ( colour == "red" ) then
		colourCode = "|cffff0000";
	elseif ( colour == "green" ) then
		colourCode = "|cff00ff00";
	elseif ( colour == "blue" ) then
		colourCode = "|cff0000ff";
	elseif ( colour == "yellow" ) then
		colourCode = "|cffffff00";
	elseif ( colour == "white" ) then
		colourCode = "|cffffffff";
	elseif ( colour == "black" ) then
		colourCode = "|cff000000";
	else
		colourCode = "|cff"..colour;
	end
        
	return colourCode..msg..FONT_COLOR_CODE_CLOSE;
end

