﻿-- **************************************************************************
-- * Titan Tokens .lua - VERSION 2.0
-- **************************************************************************

MAX_WATCHED_TOKENS = 3;

-- ******************************** Constants *******************************
local TITAN_TOKENS_ID = "Tokens";
local TITAN_TOKENS_VERSION = "0.1";
local TITAN_TOKENS_INNERPADDING = 25;
local TITAN_TOKENS_OUTTERPADDING = 15;

-- ******************************** Variables *******************************
local TitanTokens = LibStub("AceAddon-3.0"):NewAddon("TitanTokens", "AceHook-3.0", "AceTimer-3.0")
local _G = getfenv(0);
-- ******************************** Status Flags ****************************
local TITANTOKENS_INITIALIZED = false;
local TITANTOKENS_ARRAYINITIALIZED = false;
local TITANTOKENS_VARIABLES_LOADED = false;
local TITANTOKENS_ENTERINGWORLD = false;
local TOKENARRAY_INDEX = "";

-- ******************************** Functions *******************************

local tokenFrameOnUpdate = _G["TokenFrame_Update"];
-- **************************************************************************
-- NAME : TitanPanelTokensButton_OnLoad()
-- DESC : Registers the add on upon it loading
-- **************************************************************************
function TitanPanelTokensButton_OnLoad(self)
     self.registry = { 
          id = TITAN_TOKENS_ID,
          -- builtIn = nil,
          category = "Information",
          version = TITAN_TOKENS_VERSION,
          menuText = "Tokens", 
          tooltipTitle = "Tokens Info",
          tooltipTextFunction = "TitanPanelTokensButton_GetTooltipText",
          buttonTextFunction = "TitanPanelTokensButton_GetButtonText",
     };

     self:RegisterEvent("PLAYER_ENTERING_WORLD");
     self:RegisterEvent("VARIABLES_LOADED");
     self:RegisterEvent("CURRENCY_DISPLAY_UPDATE");

     _G["TokenFrame_Update"] = TitanPanelTokens_Update;
     
	if (not TokensArray) then 
		TokensArray={};
	end
     
	TitanPanelTokens_Update();

end

function TitanTokens_OnEvent(self, event, ...)
	if (event == "VARIABLES_LOADED" ) then
	      TITANTOKENS_VARIABLES_LOADED = true;
          if (TITANTOKENS_ENTERINGWORLD) then
               TitanPanelTokens_Initialize_Array(self);
          end
          return;
	end
	
	if (event == "PLAYER_ENTERING_WORLD") then
	      TITANTOKENS_ENTERINGWORLD = true;
          if (TITANTOKENS_VARIABLES_LOADED) then
               TitanPanelTokens_Initialize_Array(self);
          end
          return;
	end
	
	if (event == "CURRENCY_DISPLAY_UPDATE") then
		TitanPanelTokens_Update_Array();
		TitanPanelTokens_Update();
	end
end

function TitanPanelTokens_Initialize_Array(self)
	local name, isHeader, count, icon, itemID;
	if (TITANTOKENS_ARRAYINITIALIZED) then return; end          

	self:UnregisterEvent("VARIABLES_LOADED");
	self:UnregisterEvent("PLAYER_ENTERING_WORLD");

	TOKENARRAY_INDEX = UnitName("player").."_"..GetCVar("realmName").."::"..UnitFactionGroup("Player");

	if (not TokensArray[TOKENARRAY_INDEX]) then
		TokensArray[TOKENARRAY_INDEX] = {};
	end

	for index=1, GetCurrencyListSize() do
		name, isHeader, _isExpanded, _isUnused, _isWatched, count, _extraCurrencyType, icon, itemID = GetCurrencyListInfo(index);
		if (isHeader == false) then
			if (not TokensArray[TOKENARRAY_INDEX][index]) then
				TokensArray[TOKENARRAY_INDEX][index] = {};
			end
			
			TokensArray[TOKENARRAY_INDEX][index]["Name"] = name;
			TokensArray[TOKENARRAY_INDEX][index]["Count"] = count;
			TokensArray[TOKENARRAY_INDEX][index]["Icon"] = icon;
			TokensArray[TOKENARRAY_INDEX][index]["ItemID"] = itemID;
		end
	end

	TITANTOKENS_ARRAYINITIALIZED = true;
end

function TitanPanelTokens_Update_Array()
	local name, isHeader, count, icon, itemID;

	TOKENARRAY_INDEX = UnitName("player").."_"..GetCVar("realmName").."::"..UnitFactionGroup("Player");

	if (not TokensArray[TOKENARRAY_INDEX]) then
		TokensArray[TOKENARRAY_INDEX] = {};
	end

	for index=1, GetCurrencyListSize() do
		name, isHeader, _isExpanded, _isUnused, _isWatched, count, _extraCurrencyType, icon, itemID = GetCurrencyListInfo(index);
		if (isHeader == false) then
			if (not TokensArray[TOKENARRAY_INDEX][itemID]) then
				TokensArray[TOKENARRAY_INDEX][itemID] = {};
			end

			TokensArray[TOKENARRAY_INDEX][itemID]["Count"] = count;
		end
	end

	TITANTOKENS_ARRAYINITIALIZED = true;
end

 
-- *******************************************************************************************
-- NAME: TitanPanelTokens_Update()
-- DESC: 
-- *******************************************************************************************
function TitanPanelTokens_Update()
	local watchButton;
	local widthFrame = 0;
	local countWidth, numberDigits;
	local name, count, extraCurrencyType, icon;
	for i=1, MAX_WATCHED_TOKENS do
		name, count, extraCurrencyType, icon, itemID = GetBackpackCurrencyInfo(i);
		-- Update watched tokens
		if ( name ) then
			numberWatched = i;
			watchButton = GetClickFrame("TitanPanelTokensButtonToken"..i);
			watchButton.extraCurrencyType = extraCurrencyType;
			watchButton.count:SetText(count);
			if ( extraCurrencyType == 1 ) then	--Arena points
				watchButton.icon:SetTexture("Interface\\PVPFrame\\PVP-ArenaPoints-Icon");
				watchButton.icon:SetTexCoord(0, 1, 0, 1);
			elseif ( extraCurrencyType == 2 ) then --Honor points
				local factionGroup = UnitFactionGroup("player");
				if ( factionGroup ) then
					watchButton.icon:SetTexture("Interface\\TargetingFrame\\UI-PVP-"..factionGroup);
					watchButton.icon:SetTexCoord( 0.03125, 0.59375, 0.03125, 0.59375 );
				else
					watchButton.icon:SetTexCoord(0, 1, 0, 1);
				end
			else
				watchButton.icon:SetTexture(icon);
				watchButton.icon:SetTexCoord(0, 1, 0, 1);
			end
			
			if (count ~= 0) then
				numberDigits = floor(log10(count))+1;
			else
				numberDigits = 1
			end
			
			countWidth = numberDigits * 9;
			watchButton.count:SetWidth(countWidth);
			watchButton:SetWidth(countWidth + TITAN_TOKENS_INNERPADDING);
			widthFrame = widthFrame + countWidth + TITAN_TOKENS_INNERPADDING;
			watchButton:Show();
			watchButton.itemID = itemID;
		else
			GetClickFrame("TitanPanelTokensButtonToken"..i):Hide();
			GetClickFrame("TitanPanelTokensButtonToken"..i).itemID = nil;
		end
	end
	
	if(numberWatched ~= 0) then
		GetClickFrame("TitanPanelTokensButton"):SetWidth(widthFrame - TITAN_TOKENS_OUTTERPADDING);
	else
		GetClickFrame("TitanPanelTokensButton"):SetWidth(0);
	end
	
	if(tokenFrameOnUpdate ~= nil) then
		return tokenFrameOnUpdate();
	end
end

function TitanPanelTokensButton_GetTooltipText()
	local tooltipText = "";
	local name, count, extraCurrencyType, icon;

	for key, value in pairs(TokensArray) do
		local character, charserver, charfaction = string.match(key, '(.*)_(.*)::(.*)');

		if ((charserver == GetCVar("realmName")) and (charfaction == UnitFactionGroup("Player"))) then
		
			tooltipText = tooltipText.."\n"..character;
			
			for i=1, MAX_WATCHED_TOKENS do
				name, count, extraCurrencyType, icon, itemID = GetBackpackCurrencyInfo(i);
				if ( name ) then
					tooltipText = tooltipText.." "..value[itemID]["Count"].." ("..name..")";
				end
			end
		end
	end
	
	return tooltipText;
end

function show(tbl)
	for key,value in pairs(tbl) do
		print(key, value);
	end
end