--[[
Titan Panel [TradeCooldown] 3.1.8.30200
	By tsigo, goliathvt, totalpackage, sinage, GrayElf, Aniarc and Shad0h

	Lists tradeskill cooldowns across all characters. Displays a summary of how many cooldoans are available, and the total number being tracked on the Titan bar. Mouse-over shows a breakdown by character, and lists each skill and the time until it is available, or 'Ready!'.

	Features
		- TitanPanel module
		- At-a-glance view of the status of tradeskills with cooldowns across all characters and severs on an account.
		
	Known Issues

	Thanks
		- WoW Wiki for the excellent resource
		- The authors of previous versions
		- Otzelott for German translation.

	ChangeLog
	v3.1.8.30200 2009/10/5 - GrayElf
		- Added Void/Prismatic Sphere as 'Spheres'
		- Commented out 'Craft Updates' section, as Enchanting is now a Trade
		- TOC updated for 3.2

	v3.1.7.30100 2009/04/30 - Shad0h
		- Removed Salt Shaker
	
	v3.1.6.30100 2009/04/25 - Shad0h
		- Removed Spellcloth cooldown
		- Removed Shadowcloth cooldown
		- Removed Primal Mooncloth cooldown
		- Fixed problem with linked professions adding to character trade skills
		- Update TOC for 3.1
		
	v3.1.5.30000 2009/01/20 - Aniarc
		- Added JC Icy Prism Cooldown
		
	v3.1.4.30000 2008/12/16 - Aniarc
		- Added German translation done by Otzelott.  If you see any issues with it please let me know.
		- Smelt Titansteel tested and appears to function normally.	
		
	v3.1.3.30000 2008/12/15 - Aniarc
		- Added Smelt Titansteel to the cooldowns being monitored. 
		- ** This change is not tested since I do not have a miner. I will have to rely on the feedback **
		
	v3.1.2.30000 2008/12/15 - Aniarc
		- Added Northrend Inscription Research to be monitored 
		
	v3.1.1.30000 2008/12/14 - Aniarc
		- Minor update to title format.
		- Changed version number to follow what appears to be a new standard.
		
	v3.1 2008/12/12 - Animod
		- Removed mooncloth cooldown
		- Removed void shatter cooldown
		- Added Moonshroud, Spellweave, and Ebonweave cooldowns
		- Added Northrend Alchemy Research	
		
	v3.0 2008/10/24 - GrayElf
		- Fixed Evergreen Pouch disable option

	v3.0Beta1 2008/10/22 - GrayElf
		- Added support for WoW 3.0 and Titan Panel 4.0.
		- Merged some changes by sinage, thanks!
		
	v1.0Beta3 2008/05/9 - GrayElf
		- Try to correctly handle item cooldowns across system reboots
		- Fixed inconsistent function name for Debug function
		
	v1.0Beta2 2008/05/5 - GrayElf
		- Rewrote debugging output code
		- Added additional debugging (and removed some useless)

	v1.0Beta1 2008/05/4 - GrayElf
		- New code for item cooldowns
		- Fixed bug in Salt Shaker cooldown

	v0.44 2008/04/19 - GrayElf
		- Fixed typo in 0.43

	v0.43 2008/04/19 - GrayElf
		- Added new Jewelcrafting cooldown, "Brilliant Glass"

	v0.42 2008/04/13 - GrayElf
		- Added support for Enchanting cooldowns, starting with "Void Shatter"
		- Added debugging option, although minimal debugging output so far
		- Updated TOC for 2.4

	v0.40 2007/6/5 - GrayElf
		- Added new cooldowns thanks to daethlor
		- Bumped TOC to 2.1

	v0.36 2006/12/10 - GrayElf
		- 2.0 Compat
	
	v0.35 2006/09/18
		- Added Aglarfuin's Evergreen Pouch support and made it optional (disabled by default)
		- Updated some localization (untested), thanks worldofwar and curse community
		- Some memory/performance optimizations
		- Fixed bug (again) that cause alert to go off multiple times.
		- TOC updated for 1.12
	
	v0.33 2006/05/21
		- Fixed bug where alerts kept firing when tradeskill window is open.
		- Added option to show only current realm's cooldowns.
		- Minor tooltip reformatting.
		- Possible German support added.  Thanks Katano.
		
	v0.32 2006/04/02
		- Added Elune's Lantern
		- Disabled FR localization (need updates please!)
		- TOC updated for 1.10   
    
	v0.31 2006/02/01
		- Changed the way Snow and Salt are recorded due to possible issues from server resets
		- Added notification when ready or almost ready
		- Some French localization, will need more
            
	v0.3 01/25/06
		- Added support for the Engineer's SnowMaster 9000 cooldown by totalpackage
		- Added an option to clear all data.
		- Additional UI Tweaks.
        
	v0.21 2005/10/07
		- Added support for the Leatherworker's Salt Shaker cooldown by goliathvt

	v0.2 2005/09/23
		- Changed time() calls to the correct time(), thanks Thorarin

	v0.1 2005/09/22
		- Initial release.

]]--

-- {{{ Initialisation

-- Global Vars
TITAN_TRADECOOLDOWN_ID = "TradeCooldown";
TITAN_TRADECOOLDOWN_READY_COLOR = "|cff00FF00";
TITAN_TRADECOOLDOWN_NOTREADY_COLOR = "|cffFF0000";
TITAN_TRADECOOLDOWN_ALMOSTREADY_COLOR = "|cffF0F000";

--Time remaining to notify if the cooldown is almost ready
TITAN_TRADECOOLDOWN_NOTIFYTIME = 300;

-- Localization
-- For some stupid reason I couldn't get this to work with a localization.lua file, 
-- and yes I updated the XML file to include it.  These strings are here more as a 
-- "look to the future" and maybe later I can have translations.
TITAN_TRADECOOLDOWN_BUTTON_LABEL = "TradeCooldown: ";
TITAN_TRADECOOLDOWN_TOOLTIP_LABEL = "TradeCooldown 3.1.8.30200";
TITAN_TRADECOOLDOWN_FORMAT = "%d/%d";
TITAN_TRADECOOLDOWN_READY = "Ready!";
TITAN_TRADECOOLDOWN_SNOWMASTER = "SnowMaster 9000";
TITAN_TRADECOOLDOWN_EVERGREENPOUCH = "Evergreen Pouch";
TITAN_TRADECOOLDOWN_ELUNESLANTERN = "Elune's Lantern";
TITAN_TRADECOOLDOWN_TRANSMUTE_MATCH = "Transmute";
TITAN_TRADECOOLDOWN_TRANSMUTES = "Transmutes";
TITAN_TRADECOOLDOWN_CONFIRM = "Are you sure you want to clear all data for Titan TradeCooldown?";
TITAN_TRADECOOLDOWN_NODATA = "No Data";
TITAN_TRADECOOLDOWN_NOTIFYENABLE = "Enable Alert";
TITAN_TRADECOOLDOWN_CURRENTREALM = "Show Only Current Realm";
TITAN_TRADECOOLDOWN_SHOWEVERGREEN = "Enable Evergreen Pouch";
TITAN_TRADECOOLDOWN_CLEARALL = "Clear all data";
TITAN_TRADECOOLDOWN_CREATESEARCH = "You create (.*).";
TITAN_TRADECOOLDOWN_READYNOTIFY0 = "|cff00FF00TradeCooldown:|r %s for %s:%s is ready.";
TITAN_TRADECOOLDOWN_READYNOTIFY1 = "|cff00FF00TradeCooldown:|r %s for %s:%s will be ready in %ss.";
TITAN_TRADECOOLDOWN_WARN = "|cff00FF00TradeCooldown:|r Item created while tradeskill window is closed; mooncloth/transmutes cooldown may not update until you reopen the window.";
TITAN_TRADECOOLDOWN_SPELLWEAVE = "Spellweave";
TITAN_TRADECOOLDOWN_MOONSHROUD = "Moonshroud";
TITAN_TRADECOOLDOWN_EBONWEAVE = "Ebonweave";
TITAN_TRADECOOLDOWN_BRILLIANTGLASS = "Brilliant Glass";
TITAN_TRADECOOLDOWN_ICYPRISM = "Icy Prism";
TITAN_TRADECOOLDOWN_DEBUG = "Debug";
TITAN_TRADECOOLDOWN_SCANBAGS = "Scan Bags";
TITAN_TRADECOOLDOWN_MINORINSCRIPTION = "Minor Inscription Research";
TITAN_TRADECOOLDOWN_NORTHRENDINSCRIPTION = "Northrend Inscription Research";
TITAN_TRADECOOLDOWN_NORTHRENDALCHEMYRESEARCH = "Northrend Alchemy Research";
TITAN_TRADECOOLDOWN_SMELTTITANSTEEL = "Smelt Titansteel";
TITAN_TRADECOOLDOWN_SPHERE_MATCH = "Sphere";
TITAN_TRADECOOLDOWN_SPHERES = "Spheres";

if ( GetLocale() == "deDE" ) then
    TITAN_TRADECOOLDOWN_BUTTON_LABEL = "TradeCooldown: ";
    TITAN_TRADECOOLDOWN_TOOLTIP_LABEL = "TradeCooldown";
    TITAN_TRADECOOLDOWN_FORMAT = "%d/%d";
    TITAN_TRADECOOLDOWN_READY = "Fertig!";
    TITAN_TRADECOOLDOWN_SPELLWEAVE = "Zaubertuch";
    TITAN_TRADECOOLDOWN_MOONSHROUD = "Mondgespinst";
    TITAN_TRADECOOLDOWN_EBONWEAVE = "Schwarztuch";
    TITAN_TRADECOOLDOWN_NORTHRENDALCHEMYRESEARCH = "Alchemistische Forschung Nordends";
    TITAN_TRADECOOLDOWN_SNOWMASTER = "Schneemeister 9000";
    TITAN_TRADECOOLDOWN_EVERGREENPOUCH = "Immergr\195\188nbeutel";
    TITAN_TRADECOOLDOWN_ELUNESLANTERN = "Elunes Laterne";
    TITAN_TRADECOOLDOWN_TRANSMUTE_MATCH = "Transmutieren";
    TITAN_TRADECOOLDOWN_TRANSMUTES = "Transmutationen";
    TITAN_TRADECOOLDOWN_MINORINSCRIPTION = "Schwache Inschriftenforschung";
    TITAN_TRADECOOLDOWN_NORTHRENDINSCRIPTION = "Inschriftenforschung von Nordend";
    TITAN_TRADECOOLDOWN_CONFIRM = "Sind Sie sicher das Sie alle Daten von Titan TradeCooldown l\195\182schen wollen?";
    TITAN_TRADECOOLDOWN_NODATA = "Keine Daten";
    TITAN_TRADECOOLDOWN_CLEARALL = "Alle Daten l\195\182schen";
    TITAN_TRADECOOLDOWN_CREATESEARCH = "Ihr erschafft (.*).";
    TITAN_TRADECOOLDOWN_READYNOTIFY0 = "|cff00FF00TradeCooldown:|r %s f\195\188r %s:%s ist bereit.";
    TITAN_TRADECOOLDOWN_READYNOTIFY1 = "|cff00FF00TradeCooldown:|r %s f\195\188r %s:%s wird in %ss bereit sein.";
    TITAN_TRADECOOLDOWN_SMELTTITANSTEEL = "Titanstahl verhtten";
	
elseif ( GetLocale() == "frFR" ) then
	TITAN_TRADECOOLDOWN_BUTTON_LABEL = "TradeCooldown: ";
	TITAN_TRADECOOLDOWN_TOOLTIP_LABEL = "TradeCooldown";
	TITAN_TRADECOOLDOWN_FORMAT = "%d/%d";
	TITAN_TRADECOOLDOWN_READY = "Pr\195\170t!";
    	TITAN_TRADECOOLDOWN_SPELLWEAVE = "Spellweave";
	TITAN_TRADECOOLDOWN_MOONSHROUD = "Moonshroud";
	TITAN_TRADECOOLDOWN_EBONWEAVE = "Ebonweave";
	TITAN_TRADECOOLDOWN_NORTHRENDALCHEMYRESEARCH = "Northrend Alchemy Research";
	TITAN_TRADECOOLDOWN_SNOWMASTER = "Ma\195\174treNeige 9000";
	TITAN_TRADECOOLDOWN_EVERGREENPOUCH = "Sacoche de printemps";
	TITAN_TRADECOOLDOWN_ELUNESLANTERN = "Lanterne d'Elune";
	TITAN_TRADECOOLDOWN_TRANSMUTE_MATCH = "Transmutation";
	TITAN_TRADECOOLDOWN_TRANSMUTES = "Transmutations";
	TITAN_TRADECOOLDOWN_CONFIRM = "Etes vous s\195\187re de vouloir effacer toutes les informations de Titan TradeCooldown?";
	TITAN_TRADECOOLDOWN_NODATA = "Pas de donn\195\169es";
	TITAN_TRADECOOLDOWN_NOTIFYENABLE = "Autoriser les alertes";
	TITAN_TRADECOOLDOWN_CURRENTREALM = "Montrer seulement le royaume courant";
	TITAN_TRADECOOLDOWN_CLEARALL = "Effacer les donn\195\169es";
	TITAN_TRADECOOLDOWN_CREATESEARCH = "Vous creez (.*).";
	TITAN_TRADECOOLDOWN_READYNOTIFY0 = "|cff00FF00TradeCooldown:|r %s pour %s:%s est pr\195\170t.";
	TITAN_TRADECOOLDOWN_READYNOTIFY1 = "|cff00FF00TradeCooldown:|r %s pour %s:%s sera pr\195\170t dans %ss.";
	TITAN_TRADECOOLDOWN_WARN = "|cff00FF00TradeCooldown:|r Les cooldown des objets cre\195\169 alors que la fen\196\170tre d'artisanat est ferm\195\169 ne seront pas \195\160 jour jusqu'a rouverture de la fen\195\180tre";
end


-- Items to track cooldowns for
TTC_Items   = { TITAN_TRADECOOLDOWN_SNOWMASTER, TITAN_TRADECOOLDOWN_ELUNESLANTERN, TITAN_TRADECOOLDOWN_EVERGREENPOUCH };

--[[
-- lookup table for tradeskills whose data can be found in tradeskill windows
local tradeskills = {
	[TRANSMUTES] = true,
	[MOONSHROUD] = true,
	[SPELLWEAVE] = true,
	[EBONWEAVE] = true,
}
-- cd's that are useless in TBC and WotLK basically

-- professions where the cd is in the tradeskill window
local prof = { [TAILORING] = true, [ALCHEMY] = true, }
]]--

-- }}}
-- {{{ TitanPanelTradeCooldownButton_OnLoad()

function TitanPanelTradeCooldownButton_OnLoad(self)
--	if arg1 ~= "TitanTradeCooldown" then return end
--	self:UnregisterEvent("ADDON_LOADED")

    -- Titan Panel Registry
    self.registry = {
        id = TITAN_TRADECOOLDOWN_ID,
        menuText = TITAN_TRADECOOLDOWN_TOOLTIP_LABEL,
        buttonTextFunction = "TitanPanelTradeCooldownButton_GetButtonText",
        tooltipTitle = TITAN_TRADECOOLDOWN_TOOLTIP_LABEL,
        tooltipTextFunction = "TitanPanelTradeCooldownButton_GetTooltipText",
        category = "Profession",
        icon = "Interface\\Icons\\Spell_Nature_TimeStop.blp",
        iconWidth = 16,
        savedVariables = {
            ShowIcon = 1,
            ShowLabelText = false,
            NotifyEnable = 1,
            ShowOnlyCurrentRealm = false,
            ShowEvergreen = false,
	    Debug = false,
	    TBC = false,
	    Original = false
        }
    };
    
	self:SetScript("OnUpdate", TitanPanelTradeCooldown_OnUpdate)

    -- Events
    self:RegisterEvent("TRADE_SKILL_UPDATE");
--    self:RegisterEvent("CRAFT_UPDATE");
--    this:RegisterEvent("CHAT_MSG_SPELL_TRADESKILLS");
--    this:RegisterEvent("CHAT_MSG_LOOT");
    self:RegisterEvent("BAG_UPDATE_COOLDOWN");

    -- Variables
    TTC_SaveKey = GetCVar("realmName") .. "|" .. UnitName("player");
	currealm = GetCVar("realmName")
	TTC_Save    = {};
	TTC_Save = TTC_Save or {}
	
    TitanPanelButton_OnLoad(self);
end

-- }}}
-- {{{ TitanPanelTradeCooldown_ScanBags()
-- Scan bags for known items with cooldowns

function TitanPanelTradeCooldown_ScanBags()
	for i, itemName in ipairs(TTC_Items) do
		local startTime, cooldown, enable = GetItemCooldown(itemName);
		local currentTime = GetTime();
		TitanPanelTradeCooldown_Debug("%s: %s, %s, %s, %s", itemName, startTime, cooldown, enable, currentTime);

		if (enable == 1) then	-- Item has cooldown
			if (itemName == TITAN_TRADECOOLDOWN_EVERGREENPOUCH) then
				-- Do not update if Evergreen Pouch turned off
				if (TitanGetVar(TITAN_TRADECOOLDOWN_ID, "ShowEvergreen") ~= 1) then
				TitanPanelTradeCooldown_Debug("Evergreen pouch turned off, not checking");
					TTC_Save[TTC_SaveKey][itemName] = nil;
					break;
				end
			end

        	-- If this Realm/Player combination doesn't have a table entry, make one.
            if ( TTC_Save[TTC_SaveKey] == nil ) then
            	TTC_Save[TTC_SaveKey] = {};
            end
          
            -- By now the SaveKey entry has either been verified as existing, or has been created
            -- Do the same for the itemName entry
        	if ( TTC_Save[TTC_SaveKey][itemName] == nil ) then
            	TTC_Save[TTC_SaveKey][itemName] = {};
            end
          
            -- SaveKey and skillName tables exist
            -- Store the remaining cooldown on the skill, and a timestamp of when this was saved
            -- so we can calculate if the skill is ready or not based on what its cooldown was
            -- when we last saw it.
			if (TTC_Save[TTC_SaveKey][itemName].Cooldown ~= nil and TTC_Save[TTC_SaveKey][itemName].Cooldown + TTC_Save[TTC_SaveKey][itemName].LastCheck > time()) then
                		-- Reboot detected, how can we use this?
				if (startTime > 4000000) then
                	local diff = 4294920 - startTime + cooldown;
                	TitanPanelTradeCooldown_Debug("Reboot since cooldown detected on %s.", itemName);
                	TitanPanelTradeCooldown_Debug("Calculated %s, recorded %s cooldown.", diff, TTC_Save[TTC_SaveKey][itemName].Cooldown);
                	-- Do nothing, we already have a cooldown recorded
				end
			elseif (startTime == 0) then
				TitanPanelTradeCooldown_Debug("%s is ready now.", itemName);
				TTC_Save[TTC_SaveKey][itemName].Cooldown = 0;
				TTC_Save[TTC_SaveKey][itemName].IsReady = 1;
	            TTC_Save[TTC_SaveKey][itemName].LastCheck = time();
			elseif (startTime > currentTime) then
				TitanPanelTradeCooldown_Debug("Reboot since cooldown, unable to determine cooldown for %s.", itemName);
				TTC_Save[TTC_SaveKey][itemName] = nil;
			else
				TitanPanelTradeCooldown_Debug("%s will be ready in %ss.", itemName, startTime + cooldown - currentTime);
                TTC_Save[TTC_SaveKey][itemName].Cooldown = (startTime + cooldown) - currentTime;
	            TTC_Save[TTC_SaveKey][itemName].IsReady = 0; 
	            TTC_Save[TTC_SaveKey][itemName].LastCheck = time();
			end
		end
	end
end

-- }}}
-- {{{ TitanPanelTradeCooldownButton_OnEvent()
-- Registered event received, process given event

function TitanPanelTradeCooldownButton_OnEvent(event)
	if ( event == "BAG_UPDATE_COOLDOWN" ) then
		TitanPanelTradeCooldown_Debug("Bag cooldown update event received.");
		TitanPanelTradeCooldown_ScanBags();
	end
	
--[[
-- Enchanting is no longer a Craft, so this section is not currently used

	if ( event == "CRAFT_UPDATE" and IsTradeSkillLinked() == nil ) then
		local numCrafts = GetNumCrafts();
		TitanPanelTradeCooldown_Debug("Craft update event received, %s crafts.", numCrafts);

		for i=1, numCrafts do
			local skillName = GetCraftInfo(i);
			local cooldown = GetCraftCooldown(i);
	
			if ( cooldown ~= nil ) then
				TitanPanelTradeCooldown_Debug("Updating %s, cooldown %ss.", skillName, cooldown);
			end
		end
	end
]]--

	if ( event == "TRADE_SKILL_UPDATE" and IsTradeSkillLinked() == nil ) then
		-- This is when the TTC_Save table is populated
		
		local numSkills = GetNumTradeSkills();
		TitanPanelTradeCooldown_Debug("Trade skill update event received, %s skills", numSkills);
		
		for i=1, numSkills do
			local skillName = GetTradeSkillInfo(i);
			local cooldown = GetTradeSkillCooldown(i);

			if ( cooldown ~= nil ) then
				TitanPanelTradeCooldown_Debug("Found %s, cooldown %ss.", skillName, cooldown);
			end

			-- Condense all transmute entries into one, since they share a cooldown
			if ( cooldown ~= nil and string.find(skillName, TITAN_TRADECOOLDOWN_TRANSMUTE_MATCH) ) then
				skillName = TITAN_TRADECOOLDOWN_TRANSMUTES;
			end
			-- Condense all enchanter sphere entries into one, since they share a cooldown
			if ( cooldown ~= nil and string.find(skillName, TITAN_TRADECOOLDOWN_SPHERE_MATCH) ) then
				skillName = TITAN_TRADECOOLDOWN_SPHERES;
			end

			if ( skillName == TITAN_TRADECOOLDOWN_TRANSMUTES or skillName == TITAN_TRADECOOLDOWN_MOONSHROUD or skillName == TITAN_TRADECOOLDOWN_SPELLWEAVE or skillName == TITAN_TRADECOOLDOWN_EBONWEAVE or skillName == TITAN_TRADECOOLDOWN_BRILLIANTGLASS or skillName == TITAN_TRADECOOLDOWN_ICYPRISM or skillName == TITAN_TRADECOOLDOWN_MINORINSCRIPTION or skillName == TITAN_TRADECOOLDOWN_NORTHRENDINSCRIPTION or skillName == TITAN_TRADECOOLDOWN_NORTHRENDALCHEMYRESEARCH or skillName == TITAN_TRADECOOLDOWN_SMELTTITANSTEEL or skillName == TITAN_TRADECOOLDOWN_SPHERES ) then
				if ( cooldown == nil ) then
					cooldown = 0;
				end
				TitanPanelTradeCooldown_Debug("Updating %s, cooldown %ss.", skillName, cooldown);

				-- If this Realm/Player combination doesn't have a table entry, make one.
				if ( TTC_Save[TTC_SaveKey] == nil ) then
					TTC_Save[TTC_SaveKey] = {};
				end

				-- By now the SaveKey entry has either been verified as existing, or has been created
				-- Do the same for the skillName entry
				if ( TTC_Save[TTC_SaveKey][skillName] == nil ) then
					TTC_Save[TTC_SaveKey][skillName] = {};
				end

				-- SaveKey and skillName tables exist
				-- Store the remaining cooldown on the skill, and a timestamp of when this was saved
				-- so we can calculate if the skill is ready or not based on what its cooldown was
				-- when we last saw it.
				TTC_Save[TTC_SaveKey][skillName].Cooldown = cooldown;
				if not (TTC_Save[TTC_SaveKey][skillName].Cooldown <= 60 and TTC_Save[TTC_SaveKey][skillName].IsReady == 1) then
					TitanPanelTradeCooldown_Debug("%s will be ready in %ss.", skillName, cooldown);
					TTC_Save[TTC_SaveKey][skillName].LastCheck = time();
					TTC_Save[TTC_SaveKey][skillName].IsReady = 0; 
				else
					TitanPanelTradeCooldown_Debug("%s is ready now.", skillName);
				end
			end
		end
	end
	TitanPanelButton_UpdateButton(TITAN_TRADECOOLDOWN_ID);
	TitanPanelButton_UpdateTooltip(self);
end

-- }}}
-- {{{ TitanPanelTradeCooldownButton_GetButtonText()

function TitanPanelTradeCooldownButton_GetButtonText(id)
	local retval = "";

	local totalCount = 0;
	local readyCount = 0;

	-- Increment the total number of cooldowns, and the number of "Ready" cooldowns
	for k, v in pairs(TTC_Save) do
		local _, _, realm, player = string.find(k, "^(.+)\|(.+)$");
        
		if TitanGetVar(TITAN_TRADECOOLDOWN_ID, "ShowOnlyCurrentRealm") ~= 1 or realm == GetRealmName() then
			for skillName, skillTable in pairs(v) do
				local remaining = ((skillTable.Cooldown + skillTable.LastCheck) - time());

				totalCount = totalCount + 1;

				if ( remaining <= 0 ) then
					readyCount = readyCount + 1;
				end
			end
		end
	end

	-- Color the counts for the label text
	local colorCode = TITAN_TRADECOOLDOWN_READY_COLOR;
	if ( readyCount == 0 ) then
		colorCode = TITAN_TRADECOOLDOWN_NOTREADY_COLOR;
	end

	retval = format(colorCode .. TITAN_TRADECOOLDOWN_FORMAT .. FONT_COLOR_CODE_CLOSE, readyCount, totalCount);

	return TITAN_TRADECOOLDOWN_BUTTON_LABEL, retval;
end

-- }}}
-- {{{ TitanPanelTradeCooldownButton_GetToolTipText()

function TitanPanelTradeCooldownButton_GetTooltipText()
	local retval = "";

	for k, v in pairs(TTC_Save) do
		local _, _, realm, player = string.find(k, "^(.+)\|(.+)$");
        
		if TitanGetVar(TITAN_TRADECOOLDOWN_ID, "ShowOnlyCurrentRealm") ~= 1 or realm == GetRealmName() then
			local name = "";
			if TitanGetVar(TITAN_TRADECOOLDOWN_ID, "ShowOnlyCurrentRealm") == 1 then
				retval = retval .. player .. "\n";
			else
				retval = retval .. realm .. ": " .. player .. "\n";
			end
			for skillName, skillTable in pairs(v) do
				local remaining = ((skillTable.Cooldown + skillTable.LastCheck) - time());

				if ( remaining <= 0 ) then
					retval = retval .. TitanUtils_GetHighlightText("  "..skillName..":") .. "\t" .. TITAN_TRADECOOLDOWN_READY_COLOR .. TITAN_TRADECOOLDOWN_READY .. "|r\n";
				else
					local d,h,m,s = ChatFrame_TimeBreakDown(remaining);
					local timeString = string.format("%dd %02dh %02dm %02ds", d, h, m, s);
					if ( (d == 0) and (h == 0) ) then
						retval = retval .. TitanUtils_GetHighlightText("  "..skillName..":") .. "\t" .. TITAN_TRADECOOLDOWN_ALMOSTREADY_COLOR .. timeString .. "|r\n";
					else
						retval = retval .. TitanUtils_GetHighlightText("  "..skillName..":") .. "\t" .. TITAN_TRADECOOLDOWN_NOTREADY_COLOR .. timeString .. "|r\n";
					end
				end
			end
		end

	end
	if( retval == "" ) then
		return TITAN_TRADECOOLDOWN_NODATA.."\n";
	else
		return retval;
	end
end

-- }}}

function TitanPanelRightClickMenu_PrepareTradeCooldownMenu()
	TitanPanelRightClickMenu_AddTitle(TITAN_TRADECOOLDOWN_TOOLTIP_LABEL);
	TitanPanelRightClickMenu_AddToggleIcon(TITAN_TRADECOOLDOWN_ID);
	TitanPanelRightClickMenu_AddToggleLabelText(TITAN_TRADECOOLDOWN_ID);

	TitanPanelRightClickMenu_AddSpacer();
	info = {};
	info.text = TITAN_TRADECOOLDOWN_NOTIFYENABLE;
	info.func = TitanPanelTradeCooldown_ToggleNotify;
	info.checked = TitanGetVar(TITAN_TRADECOOLDOWN_ID, "NotifyEnable");
	info.keepShownOnClick = 1;
	UIDropDownMenu_AddButton(info);
	
	info = {};
	info.text = TITAN_TRADECOOLDOWN_CURRENTREALM;
	info.func = TitanPanelTradeCooldown_ToggleRealm;
	info.checked = TitanGetVar(TITAN_TRADECOOLDOWN_ID, "ShowOnlyCurrentRealm");
	info.keepShownOnClick = 1;
	UIDropDownMenu_AddButton(info);
	
	info = {};
	info.text = TITAN_TRADECOOLDOWN_SHOWEVERGREEN;
	info.func = TitanPanelTradeCooldown_ToggleEvergreen;
	info.checked = TitanGetVar(TITAN_TRADECOOLDOWN_ID, "ShowEvergreen");
	info.keepShownOnClick = 1;
	UIDropDownMenu_AddButton(info);

	TitanPanelRightClickMenu_AddSpacer();
	info = {};
	info.text = TITAN_TRADECOOLDOWN_CLEARALL;
	info.func = TitanPanelTradeCooldown_ClearConfirm;
	UIDropDownMenu_AddButton(info);
	
	info = {};
	info.text = TITAN_TRADECOOLDOWN_SCANBAGS;
	info.func = TitanPanelTradeCooldown_ScanBags;
	UIDropDownMenu_AddButton(info);
	
	info = {};
	info.text = TITAN_TRADECOOLDOWN_DEBUG;
	info.func = TitanPanelTradeCooldown_ToggleDebug;
	info.checked = TitanGetVar(TITAN_TRADECOOLDOWN_ID, "Debug");
	info.keepShownOnClick = 1;
	UIDropDownMenu_AddButton(info);
	
	TitanPanelRightClickMenu_AddSpacer();
	TitanPanelRightClickMenu_AddCommand(TITAN_PANEL_MENU_HIDE, TITAN_TRADECOOLDOWN_ID, TITAN_PANEL_MENU_FUNC_HIDE);
end

function TitanPanelTradeCooldown_Clear()
	TitanPanelTradeCooldown_Debug("Clearing all saved data.");
	TTC_Save = {};
	TitanPanelButton_UpdateButton(TITAN_TRADECOOLDOWN_ID);
end

function TitanPanelTradeCooldown_ClearConfirm()
	StaticPopupDialogs["TITANPANELTRADECOOLDOWN_CLEAR"] = {
		text = TEXT(TITAN_TRADECOOLDOWN_CONFIRM),
		button1 = TEXT(OKAY),
		button2 = TEXT(CANCEL),
		OnAccept = function()
			TitanPanelTradeCooldown_Clear();
		end,
		timeout = 0,
		exclusive = 1
	};
	StaticPopup_Show("TITANPANELTRADECOOLDOWN_CLEAR");
end

local tcelapsed = 0;
function TitanPanelTradeCooldown_OnUpdate(self, elapsed)
	tcelapsed = tcelapsed + elapsed;
	if tcelapsed < 5 then
		return;
	end
	tcelapsed = 0;
	TitanPanelButton_UpdateButton(TITAN_TRADECOOLDOWN_ID);
	TitanPanelButton_UpdateTooltip(self);
	
	if( TitanGetVar(TITAN_TRADECOOLDOWN_ID, "NotifyEnable") == 1 ) then
		for k, v in pairs(TTC_Save) do
			local _, _, realm, player = string.find(k, "^(.+)\|(.+)$");
			if TitanGetVar(TITAN_TRADECOOLDOWN_ID, "ShowOnlyCurrentRealm") ~= 1 or realm == GetRealmName() then
				for skillName, skillTable in pairs(v) do
					local remaining = ((skillTable.Cooldown + skillTable.LastCheck) - time());

					if (remaining <= TITAN_TRADECOOLDOWN_NOTIFYTIME) then
						if (TTC_Save[k][skillName].IsReady ~= 1) then
	        					local _, _, realm, player = string.find(k, "^(.+)\|(.+)$");
	        					if (remaining <= 0) then
								DEFAULT_CHAT_FRAME:AddMessage(format(TITAN_TRADECOOLDOWN_READYNOTIFY0, skillName, realm, player)); 
							else
								DEFAULT_CHAT_FRAME:AddMessage(format(TITAN_TRADECOOLDOWN_READYNOTIFY1, skillName, realm, player, floor(remaining+0.9))); 
							end
							PlaySound("AuctionWindowOpen");
							TTC_Save[k][skillName].IsReady = 1;
						end
					end
				end
			end
		end
	end
end


function TitanPanelTradeCooldown_ToggleNotify()
	TitanPanelTradeCooldown_Debug("Toggling notifications setting.");
	TitanToggleVar(TITAN_TRADECOOLDOWN_ID, "NotifyEnable");
end

function TitanPanelTradeCooldown_ToggleRealm()
	TitanPanelTradeCooldown_Debug("Toggling current realm only setting.");
	TitanToggleVar(TITAN_TRADECOOLDOWN_ID, "ShowOnlyCurrentRealm");
	TitanPanelButton_UpdateButton(TITAN_TRADECOOLDOWN_ID);
end

function TitanPanelTradeCooldown_ToggleEvergreen()
	TitanPanelTradeCooldown_Debug("Toggling evergreen pouch setting.");
	TitanToggleVar(TITAN_TRADECOOLDOWN_ID, "ShowEvergreen");
	TitanPanelButton_UpdateButton(TITAN_TRADECOOLDOWN_ID);
end

local function ToggleShowOld()
	TitanPanelTradeCooldown_Debug("Toggling showing old cooldowns.");
	TitanToggleVar(TITAN_TRADECOOLDOWN_ID, "ShowOld");
	TitanPanelButton_UpdateButton(TITAN_TRADECOOLDOWN_ID);
end

function TitanPanelTradeCooldown_ToggleDebug()
	TitanPanelTradeCooldown_Debug("Toggling debug setting.");
	TitanToggleVar(TITAN_TRADECOOLDOWN_ID, "Debug");
	TitanPanelButton_UpdateButton(TITAN_TRADECOOLDOWN_ID);
end

function TitanPanelTradeCooldown_Debug(fmt, ...)
	if ( TitanGetVar(TITAN_TRADECOOLDOWN_ID, "Debug") == 1 ) then
			-- DEFAULT_CHAT_FRAME:AddMessage("TTC: Craft update event received.");
		if (...) then
			DEFAULT_CHAT_FRAME:AddMessage(format("TTC: "..fmt, ...));
		else
			DEFAULT_CHAT_FRAME:AddMessage("TTC: "..fmt);
		end
	end
end
